package com.playmore.util.snowflake;

import com.playmore.util.snowflake.Sequence;
import com.playmore.util.snowflake.SystemClock;
import java.util.HashSet;

public class SnowflakeUtil implements Sequence {

   private static final long twepoch = 1533052800000L;
   private static final long workerIdBits = 7L;
   private static final long sequenceBits = 2L;
   private static final long timestampBits = 38L;
   private static final long timestampShift = 2L;
   private static final long workerShift = 40L;
   private static final long highestShift = 47L;
   private static final long timestampMask = 274877906943L;
   private static final long workerIdMask = 127L;
   private static final long sequenceMask = 3L;
   private long lastTimestamp = -1L;
   private long sequence = 0L;
   private final Long workerId;


   public SnowflakeUtil(long workerId) {
      if(workerId <= 127L && workerId >= 0L) {
         this.workerId = Long.valueOf(workerId);
      } else {
         throw new IllegalArgumentException(String.format("worker Id can\'t be greater than %d or less than 0", new Object[]{Long.valueOf(127L)}));
      }
   }

   public synchronized long next() {
      long timestamp = this.currentTimestamp();
      if(timestamp == this.lastTimestamp) {
         this.sequence = this.sequence + 1L & 3L;
         if(this.sequence == 0L) {
            timestamp = this.untilNextMillis(this.lastTimestamp);
         }
      } else {
         this.sequence = 0L;
      }

      this.lastTimestamp = timestamp;
      long disTime = timestamp - 1533052800000L;
      long currentSequenceId = 140737488355328L | (this.workerId.longValue() & 127L) << 40 | (disTime & 274877906943L) << 2 | this.sequence & 3L;
      return currentSequenceId;
   }

   protected long untilNextMillis(long lastTimestamp) {
      long timestamp;
      for(timestamp = this.currentTimestamp(); timestamp <= lastTimestamp; timestamp = this.currentTimestamp()) {
         ;
      }

      return timestamp;
   }

   private long currentTimestamp() {
      return SystemClock.currentTimestamp();
   }

   public static void main(String[] args) {
      byte serverId = 125;
      long now = System.currentTimeMillis();
      HashSet ids = new HashSet();
      SnowflakeUtil sequence = new SnowflakeUtil((long)serverId);

      for(int v = 0; v < 100; ++v) {
         long id = sequence.next();
         if(ids.contains(Long.valueOf(id))) {
            System.err.println("重复id：" + id + "  i=" + v);
            break;
         }

         ids.add(Long.valueOf(id));
         System.out.println(id);
      }

      long var9 = 252288000000L;
      System.out.println("---v=" + var9);
      System.out.println("耗时=" + (System.currentTimeMillis() - now));
      System.out.println("maxTimestamp=274877906943");
   }
}
